interval 60
always_start true

critical 70
warning 85

collect 'linux' do
  m = File.read('/proc/meminfo').split(/\n/).inject({}) do |info, line|
    x = line.split(/:?\s+/)
    info[x[0]] = x[1].to_i
    info
  end

  free = m['MemFree'].to_i * 1024
  cached = m['Cached'].to_i * 1024
  buffers = m['Buffers'].to_i * 1024
  total = m['MemTotal'].to_i * 1024
  used = total - free
  free_bc = free + buffers + cached
  fraction = 1 - (free_bc.to_f / total)
  swap_fraction = m['SwapTotal'] == 0 ? 0 : 1 - m['SwapFree'].to_f/m['SwapTotal']

  event(:service => 'memory % free', :description => 'Memory usage, %', :metric => fraction.round(2) * 100)
  event(:service => 'memory % swap', :description => 'Swap usage, %', :metric => swap_fraction.round(2) * 100)
  event(:service => 'memory abs free', :description => 'Memory free (kB)', :metric => free, :state => 'ok')
  event(:service => 'memory abs total', :description => 'Memory total (kB)', :metric => total, :state => 'ok')
  event(:service => 'memory abs cached', :description => 'Memory usage, cached (kB)', :metric => cached, :state => 'ok')
  event(:service => 'memory abs buffers', :description => 'Memory usage, buffers (kB)', :metric => buffers, :state => 'ok')
  event(:service => 'memory abs used', :description => 'Memory usage, used (kB)', :metric => used, :state => 'ok')
  event(:service => 'memory abs free_bc', :description => 'Memory usage with cache and buffers (kB)', :metric => free_bc, :state => 'ok')
end
