interval 60

warning 120 # replication lag
critical 500 # replication lag
settings :host, '127.0.0.1'
settings :user, 'postgres'
settings :pgsql, '/usr/bin/psql'
settings :db4monit, 'riemann_monit'
settings :conn_warn, 5 # reserved pool connections
settings :conn_crit, 3 # reserved pool connections

run_if 'linux' do
  File.exists? settings.pgsql
end

collect 'linux' do

  # helpers
  def run_sql(sql, db='postgres')
    shell_out!("#{settings.psql} -h #{settings.host} -U #{settings.user} -tnc \"#{sql}\" #{db}").stdout
  end

  def in_recovery?
    run_sql('select pg_is_in_recovery()') == 't'
  end

  def db4monit_exists?
    run_sql("select 1 from pg_database where datname = '#{settings.db4monit}'") == '1'
  end

  def run_master_sql
    run_sql("create database #{settings.db4monit}") unless db4monit_exists?
    run_sql(
        "drop table if exists timestamp; \
      create table timestamp ( id int primary key, value timestamp default now() ); \
      insert into timestamp (id) values (1); \
        ", settings.db4monit)
  end

  def repl_lag
    unixnow - run_sql('select extract(epoch from value::timestamp) from timestamp where id = 1;', settings.db4monit).to_i
  end

  def connections
    max_conn = run_sql('show max_connections').to_i
    res_conn = run_sql('show superuser_reserved_connections').to_i
    cur_conn = run_sql('select count(1) from pg_stat_activity;').to_i
    [cur_conn, (max_conn - res_conn - cur_conn)]
  end

  # check status

  cur_conn, res_conn = connections
  if in_recovery?
    event(:service => 'pgsql replication lag', :description => 'Postgresql replication lag', :metric => repl_lag)
  else
    run_master_sql
  end

  event(:service => 'pgsql connections', :description => 'Postgresql current connections', :state => 'ok', :metric => cur_conn)

  # check reserved pool size
  if res_conn < settings.conn_warn
    if res_conn > settings.conn_crit
      event(:service => 'pgsql reserved connections', :description => 'Postgresql reserved connections state', :state => 'warning', :metric => res_conn)
    else
      event(:service => 'pgsql reserved connections', :description => 'Postgresql reserved connections state', :state => 'critical', :metric => res_conn)
    end
  end

end
