require 'malevich/loader'

module Malevich
  class Kernel

    include Malevich::Loader

    attr_reader :config, :cmd, :plugins, :ohai, :monitor, :riemann_events, :dsl

    def initialize
      @config = Malevich::Hashie.new # for config from file
      @cmd = Malevich::Hashie.new
      @plugins = Malevich::Hashie.new # for plugin containers (errors, settings)
      @logger = Logger.new(STDOUT)
      @monitor = Malevich::Monitor.new
      @riemann_events = ::Queue.new
    end

    def test_plugin(file)
      return unless malevich.cmd[:test_given]
      Malevich::DSL.test(file).run!
      exit 0
    end

    def log(level = :info, message)
      @logger.send(level.to_sym, message)
    end

    def log_level=(level = 'INFO')
      @logger.level = Logger.const_get(level)
    end

    def ohai
      unless @ohai
        Kernel::log :info, 'Load ohai data'
        @ohai = {}
        @ohai[:os] = 'mac_os_x'
        @ohai[:platform] = 'mac_os_x'
      end
      @ohai
    end
  end

  module Mixin

    class << self; attr_accessor :instance; end

    def malevich
      @instance ||= Malevich::Mixin.instance ||= Malevich::Kernel.new
    end

    def log(level = :info, message)
      speaker = self.class == Class ? "C: #{self.to_s}" : "I: #{self.class.to_s}"
      malevich.log(level, "[#{speaker}] #{message}")
    end

  end

end

require 'malevich/init'
