module Malevich
  class DSL

    PLUGIN_EXT = '.rb'

    attr_reader :plugins

    def initialize
      @plugins = Array.new
    end

    def interval(val)
      plugins.last.interval = val.to_f
    end

    def always_start(val)
      plugins.last.always_start = !!val
    end

    alias :auto :always_start
    alias :auto_start :always_start

    def run_if(*name, &block)
      return unless plugins.last.suitable_platform?(name)
      plugins.last.run_if = block
    end

    def state(attr, val)
      plugins.last.settings[attr] = val
    end

    alias :states :state

    def critical(val)
      plugins.last.settings[:critical] = val
    end

    alias :critical= :critical

    def warning(val)
      plugins.last.settings[:warning] = val
    end

    alias :warning= :warning

    def settings(attr, val)
      plugins.last.settings[attr] = val
    end

    alias :config :settings
    alias :defaults :settings
    alias :plugin :settings

    def collect(*name, &block)
      return unless plugins.last.suitable_platform?(name)
      plugins.last.collect = block
    end

    def self.load(paths)
      dsl = Malevich::DSL.new
      paths.map do |path|
        File.directory?(path) ? Dir["#{path}/*#{PLUGIN_EXT}"].sort : path
      end.flatten.each do |path|
        begin
          log :debug, "Load plugin #{path}"
          dsl.plugins << Malevich::Plugin.new(File.basename(path, PLUGIN_EXT))
          dsl.instance_eval(File.read(path), path)
        rescue ScriptError
          dsl.plugins.pop # todo: plugin.new creates
          log :error, "ScriptError: plugin from file #{path}"
        end
      end
      dsl.plugins
    end

    def self.test(file)
      dsl = Malevich::DSL.new
      dsl.plugins << Malevich::Plugin.new(File.basename(file, PLUGIN_EXT))
      dsl.instance_eval(File.read(file), file)
      dsl.plugins[0]
    end

  end
end
