require 'ruby-prof'
RubyProf.measure_mode = RubyProf::MEMORY

module Malevich
  class Monitor

    CHECK_ALIVE = 5

    attr_reader :tasks

    def initialize
      @tasks = Array.new
    end

    def plugins
      @tasks.select { |t| t[1].is_a?(Malevich::Plugin) }.map { |x| {x[1].name => x[1].settings.to_hash} }
    end

    def <<(obj)
      return false unless obj.respond_to?(:name) || obj.respond_to?(:run!)
      th = Thread.new do
        obj.run!
      end
      log :info, "Add '#{obj.class}(#{obj.name})'"
      @tasks << [th, obj]
    end

    def run_gc?
      @gc_counter ||= 0
      @gc_counter += 1
      @gc_counter > 100 ? @gc_counter = 0 : false
    end

    def run!
      loop do
        @tasks.each_with_index do |task, i|
          next if task[0].alive?
          # start new thread
          log :error, "Thread for '#{task[1].class}(#{task[1].name})' is dead, start it"
          @tasks.delete_at(i)
          self << task[1]
        end
        log :debug, "Check alive #{@tasks.count} threads"
        ObjectSpace.garbage_collect if run_gc?
        sleep CHECK_ALIVE
      end
    end

  end
end
