require_relative 'plugin/init'

module Malevich
  class Plugin

    include Malevich::PluginHelpers
    include Malevich::Mixin

    attr_accessor :name, :always_start, :interval,
                  :run_if, :collect, :settings, :histories, :errors

    alias :plugin :settings

    def initialize(name)
      @name = name
      @interval = 60
      @run_if = Proc.new { true }
      @always_start = false
    end

    def suitable_platform?(name)
      request_platform = name.nil? || name.empty? ? ['linux'] : name
      request_platform.include?(ohai[:platform]) || request_platform.include?(ohai[:os])
    end

    def runnable?
      def log_and_false(level = :info, msg)
        log level, msg
        false
      end

      return log_and_false("'#{name}' not started, because have not 'collect'") if collect.nil?
      return log_and_false("'#{name}' disabled in config") if settings.disable
      return log_and_false("'#{name} disabled by run_if statement") unless !!self.instance_eval(&run_if)
      log :unknown, "'#{name}' started"
    end

    def run!
      loop do
        t_start = Time.now
        begin
          Timeout.timeout(interval.to_f * 2/3) do
            self.instance_eval(&collect)
          end
        rescue => e
          error(e)
        end
        sleep(interval - (Time.now - t_start).to_i)
      end
    end

    private

    def ohai
      malevich.ohai
    end

    def self.make_container(container)
      define_method(container) do |&block|
        malevich.plugins[container] ||= Malevich::Hashie.new
        malevich.plugins[container][self.name] ||= Malevich::Hashie.new
        malevich.plugins[container][self.name]
      end
    end

    make_container :settings
    alias :plugin :settings
    make_container :histories
    make_container :errors

  end
end
