module Malevich
  module PluginHelpers

    def event(hash)
      event_normalize(hash)
      hash[:metric] = metric_diff(hash) if hash[:diff]
      hash[:state] = state_check(hash)
      if malevich.cmd[:test_given]
        log :unknown, "Event message from test plugin: #{hash.inspect}"
      else
        event_minimizer(hash)
      end
    end

    def metric_diff(hash)
      return unless hash[:metric]
      current_metric = hash[:metric]
      old_metric = histories[hash[:service]] ? histories[hash[:service]][:metric] : nil
      histories[hash[:service]] = current_metric
      hash.delete(:diff)
      old_metric ? current_metric - old_metric : nil
    end

    def state_check(hash)
      return hash[:state] if hash[:state]
      return hash[:state] if hash[:metric].nil?
      return hash[:state] if hash[:metric].kind_of?(Float) && hash[:metric].nan?
      warning = settings.respond_to?(:warning) ? settings.warning : nil
      critical = settings.respond_to?(:critical) ? settings.critical : nil
      return 'ok' if (warning || critical).nil?
      metric = hash[:metric].to_f
      if warning && critical
        return case
                 when metric.between?(warning, critical)
                   'warning'
                 when metric > warning
                   'critical'
                 else
                   'ok'
               end
      end
      if warning
        return (metric >= warning) ? 'warning' : 'ok'
      end
      if critical
        return (metric >= critical) ? 'critical' : 'ok'
      end
      'critical'
    end

    def event_normalize(hash)
      hash[:metric] = hash[:metric].round(2) if hash[:metric].kind_of?(Float)
      hash[:state] = 'ok' if hash[:state].kind_of?(TrueClass)
      hash[:state] = 'critical' if hash[:state].kind_of?(FalseClass)
      hash[:service] ||= name
      hash[:host] ||= ohai[:fqdn]
      hash[:tags] ||= malevich.cmd.tags
    end

    def event_minimizer(hash)
      return if hash[:state] == 'ok' && hash[:metric].nil? && histories[hash[:service]] &&
          histories[hash[:service]][:state] == 'ok'
      histories[hash[:service]] = hash
      malevich.riemann_events << hash
    end

  end
end
