require 'socket'

module Malevich
  module Responder
    class Udp

      include Malevich::Mixin

      RIEMANN_RESERVED_FIELDS = [
          :time,
          :state,
          :service,
          :host,
          :description,
          :metric,
          :tags,
          :ttl
      ]

      attr_reader :name

      def initialize
        @host, @port = malevich.cmd[:'udp-responder'].split(':')
        @name = 'udp api'
      end

      def process(data, src)
        begin
          malevich.riemann_events << event_from_json(data)
          src.reply "sended\n\n"
        rescue
          log :error, "Failed to send message: #{data.inspect}"
          src.reply "failed to send: #{data.inspect}\n"
        end
      end

      def event_from_hash(hash=nil)
        new_hash = Hash.new
        RIEMANN_RESERVED_FIELDS.each do |key|
          new_hash[key] = hash[key] || hash[key.to_s]
        end
        new_hash[:host] ||= malevich.ohai[:fqdn]
        new_hash[:tags] ||= malevich.cmd.tags
        new_hash
      end

      def event_from_json(str)
        event_from_hash(JSON.parse(str))
      end

      def run!
        log :unknown, "Start udp server at #{@host}:#{@port}"
        Socket.udp_server_loop(@host, @port) do |data, src|
          log :debug, "Received data: #{data.inspect}, from client: #{src.inspect}"
          process(data, src)
        end
      end

    end
  end
end
